==> comphep-4.4.113.tgz: comphep-4.4.113/src/num/mix_format_lhef.c <==
/*
* Copyright (C) 2007-2008, CompHEP Collaboration
* Author: Alexander Sherstnev 
* ------------------------------------------------------
* $Id$
*
* $Log$
*/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include "service2/include/chep_limits.h"
#include "service2/include/syst.h"

#include "mix_format_lhef.h"
#include "lhef_routines.h"

int mix_format_lhef (int nf, const char names[], const int lenth)
{

  int i, j;
  int mf;
  int final;
  int nRecotot;
  int nLefttot;
  int *nLeft;
  int *file;
  int *xmlerr;

  long endpos;
  long **pos;
  long *pos0;
  long *wpos;

  double *ri;
  double delta;
  double *CSec;
  double *CSerr;
  double xrn;
  double errtot;
  double sigmatot;
  char **map;
  char **filename;

  FILE** infile;
  FILE* outFile;
  FILE* prtFile;

  char* outFileName = "Mixed.PEV";
  char* prtFileName = "prt.pev";

  filename = malloc (nf * sizeof(char*));
  infile   = malloc (nf * sizeof(FILE*));

  pos0   = malloc (nf * sizeof (long));
  pos    = malloc (nf * sizeof (long *));
  CSec   = malloc (nf * sizeof (double));
  CSerr  = malloc (nf * sizeof (double));
  ri     = malloc (nf * sizeof (double));
  map    = malloc (nf * sizeof (char *));
  nLeft  = malloc (nf * sizeof (int));
  xmlerr = malloc (nf * sizeof (int));

  prtFile = fopen (prtFileName, "w");
  if (NULL == prtFile) {
    fprintf (stderr, "mix: Error!");
    fprintf (prtFile, "Files to mix and randomize:\n");
  }

  for (i = 0; i < nf; ++i) {
    filename[i] = malloc ((lenth + 1) * sizeof(char));
    strncpy (filename[i], names + i * lenth, lenth);
    trim (filename[i]);
    infile[i] = fopen (filename[i], "r");
    if (!infile[i]) {
      fprintf (stderr, "File not found: %s\n", filename[i]);
      fprintf (prtFile, "File not found: %s\n", filename[i]);
      return 2;
    } else {
      fprintf (prtFile, "%s\n", filename[i]);
    }
    fclose (infile[i]);
  }

  for (i = 0; i < nf; ++i) {
    xmlerr[i] = formXMLtree (filename[i], i);
    pos0[i]   = getEventPosition (i);
    nLeft[i]  = getEventNumber (i);
    CSec[i]   = getCrossSection (i);
    CSerr[i]  = getCrossSectionErr (i);
  }

  mf = nf;
  for (i = nf; i > 0; --i) {
    if (0 > xmlerr[i] || 0 > pos0[i] || 0 > nLeft[i] || 0 > CSec[i]) {
      --mf;
      for (j = i; i < mf; ++i) {
        pos0[j]  = pos0 [j + 1];
        nLeft[j] = nLeft [j + 1];
        CSec[j]  = CSec [j + 1];
        strcpy (filename[j], filename[j + 1]);
      }
    }
  }
  nf = mf;

  nLefttot = 0;
  sigmatot = 0.0;
  errtot = 0.0;
  for (i = 0; i < nf; ++i) {
    int num = 0;
    infile[i] = fopen (filename[i], "r");
    nLefttot += nLeft[i];
    sigmatot += CSec[i];
    delta = 0.0;
    if (0.0 < CSec[i]) delta = CSerr[i]/CSec[i];
    errtot = delta*delta;
    pos[i] = malloc (nLeft[i] * sizeof (long));
    map[i] = malloc (nLeft[i] * sizeof (char));
    pos[i][num] = pos0[i];
    map[i][num] = 0;
    while (!testEvent (infile[i], filename[i], pos[i][num])) {
      ++num;
      pos[i][num] = ftell (infile[i]);
      map[i][num] = 0;
    }
    if (num != nLeft[i]) {
      fprintf (stderr, "mix: (warning) the real number of events (%i) in file %s does not correspond reporting number (%i) in header.\n", num, filename[i], nLeft[i]);
      nLeft[i] = num;
    }
  }
  errtot = sigmatot * sqrt (errtot);

  xrn = 0.0;
  for (i = 0; i < nf; i++) {
    ri[i] = xrn + CSec[i] / sigmatot;
    xrn = ri[i];
  }

  wpos = malloc (nLefttot * sizeof (long));
  file = malloc (nLefttot * sizeof (int));
  nRecotot = 0;
  while (nRecotot < nLefttot) {
    xrn = drand48 ();
    i = 0;
    while (xrn > ri[i]) {
      i++;
    }
    file[nRecotot] = i;

    if (0 >= nLeft[i] ) {
      final = i;
      break;
    }

    j = 0;
    while (map[i][j]) {
      ++j;
    }
    map[i][j] = 1;
    wpos[nRecotot] = pos[i][j];
    ++nRecotot;
    --(nLeft[i]);
  }

  set_cs (sigmatot, errtot);
  endpos = write_event_header_libxml2 (outFileName, "w");
  outFile = fopen (outFileName, "a");
  for (i = 0; i < nf; ++i) {
    infile[i] = freopen (filename[i], "r", infile[i]);
  }
  for (i = 0; i < nRecotot; ++i) {
    moveEvent (filename[file[i]], infile[file[i]], wpos[i], outFile, endpos);
    endpos = ftell(outFile);
  }

  fprintf (stdout,  "mix: file %s exhausted!\n", filename [final]);
  fprintf (stdout,  "mix: %i mixed/randomized events written to Mixed.PEV\n", nRecotot);
  fprintf (prtFile, "mix: %i mixed/randomized events written to Mixed.PEV\n", nRecotot);
  fclose (outFile);
  fclose (prtFile);
  {
    int filesize = 0;
    char check_sum[128];
    struct stat stt;
    char command[1024];

    stat (outFileName, &stt);
    filesize = stt.st_size;
    for (i = 0; i < 128; ++i) check_sum[i] = 0;
    sprintf (command, "md5sum %s > md5sum.log", outFileName);
    system(command);
    FILE * md5 = fopen ("md5sum.log", "r");
    if (md5) {
      fscanf (md5, "%s", check_sum);
    }
    set_final_numbers (nRecotot, filesize, sigmatot, errtot, check_sum);
    write_file_header_libxml2 (outFileName, "r+");
  }

  for (i = 0; i < nf; ++i) {
    free (filename[i]);
    free (pos[i]);
    free (map[i]);
  }
  free (filename);
  free (infile);
  free (pos);
  free (map);
  free (pos0);
  free (CSec);
  free (nLeft);
  free (ri);
  free (wpos);
  free (file);
  free (xmlerr);

  return 1;
}

==> comphep-4.5.0rc1.tgz: comphep-4.5.0rc1/src/num/mix_format_lhef.c <==
/*
* Copyright (C) 2007-2008, CompHEP Collaboration
* Author: Alexander Sherstnev 
* ------------------------------------------------------
* $Id$
*
* $Log$
*/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

#include "service2/include/chep_limits.h"
#include "service2/include/syst.h"

#include "mix_format_lhef.h"
#include "lhef_routines.h"

int mix_format_lhef (int nf, const char names[], const int lenth)
{

  int i, j;
  int mf;
  int final;
  int nRecotot;
  int nLefttot;
  int *nLeft;
  int *file;
  int *xmlerr;

  long endpos;
  long **pos;
  long *pos0;
  long *wpos;

  double *ri;
  double delta;
  double *CSec;
  double *CSerr;
  double xrn;
  double errtot;
  double sigmatot;
  char **map;
  char **filename;

  FILE** infile;
  FILE* outFile;
  FILE* prtFile;

  char* outFileName = "Mixed.PEV";
  char* prtFileName = "prt.pev";

  filename = malloc (nf * sizeof(char*));
  infile   = malloc (nf * sizeof(FILE*));

  pos0   = malloc (nf * sizeof (long));
  pos    = malloc (nf * sizeof (long *));
  CSec   = malloc (nf * sizeof (double));
  CSerr  = malloc (nf * sizeof (double));
  ri     = malloc (nf * sizeof (double));
  map    = malloc (nf * sizeof (char *));
  nLeft  = malloc (nf * sizeof (int));
  xmlerr = malloc (nf * sizeof (int));

  prtFile = fopen (prtFileName, "w");
  if (NULL == prtFile) {
    fprintf (stderr, "mix: Error!");
    fprintf (prtFile, "Files to mix and randomize:\n");
  }

  for (i = 0; i < nf; ++i) {
    filename[i] = malloc ((lenth + 1) * sizeof(char));
    strncpy (filename[i], names + i * lenth, lenth);
    trim (filename[i]);
    infile[i] = fopen (filename[i], "r");
    if (!infile[i]) {
      fprintf (stderr, "File not found: %s\n", filename[i]);
      fprintf (prtFile, "File not found: %s\n", filename[i]);
      return 2;
    } else {
      fprintf (prtFile, "%s\n", filename[i]);
    }
    fclose (infile[i]);
  }

  for (i = 0; i < nf; ++i) {
    xmlerr[i] = formXMLtree (filename[i], i);
    pos0[i]   = getEventPosition (i);
    nLeft[i]  = getEventNumber (i);
    CSec[i]   = getCrossSection (i);
    CSerr[i]  = getCrossSectionErr (i);
  }

  mf = nf;
  for (i = nf; i > 0; --i) {
    if (0 > xmlerr[i] || 0 > pos0[i] || 0 > nLeft[i] || 0 > CSec[i]) {
      --mf;
      for (j = i; i < mf; ++i) {
        pos0[j]  = pos0 [j + 1];
        nLeft[j] = nLeft [j + 1];
        CSec[j]  = CSec [j + 1];
        strcpy (filename[j], filename[j + 1]);
      }
    }
  }
  nf = mf;

  nLefttot = 0;
  sigmatot = 0.0;
  errtot = 0.0;
  for (i = 0; i < nf; ++i) {
    int num = 0;
    infile[i] = fopen (filename[i], "r");
    nLefttot += nLeft[i];
    sigmatot += CSec[i];
    delta = 0.0;
    if (0.0 < CSec[i]) delta = CSerr[i]/CSec[i];
    errtot = delta*delta;
    pos[i] = malloc (nLeft[i] * sizeof (long));
    map[i] = malloc (nLeft[i] * sizeof (char));
    pos[i][num] = pos0[i];
    map[i][num] = 0;
    while (!testEvent (infile[i], filename[i], pos[i][num])) {
      ++num;
      pos[i][num] = ftell (infile[i]);
      map[i][num] = 0;
    }
    if (num != nLeft[i]) {
      fprintf (stderr, "mix (warning): the real number of events (%i) in file %s does not correspond reporting number (%i) in header.\n", num, filename[i], nLeft[i]);
      nLeft[i] = num;
    }
  }
  errtot = sigmatot * sqrt (errtot);

  xrn = 0.0;
  for (i = 0; i < nf; i++) {
    ri[i] = xrn + CSec[i] / sigmatot;
    xrn = ri[i];
  }

  wpos = malloc (nLefttot * sizeof (long));
  file = malloc (nLefttot * sizeof (int));
  nRecotot = 0;
  while (nRecotot < nLefttot) {
    xrn = drand48 ();
    i = 0;
    while (xrn > ri[i]) {
      i++;
    }
    file[nRecotot] = i;

    if (0 >= nLeft[i] ) {
      final = i;
      break;
    }

    j = 0;
    while (map[i][j]) {
      ++j;
    }
    map[i][j] = 1;
    wpos[nRecotot] = pos[i][j];
    ++nRecotot;
    --(nLeft[i]);
  }

  set_cs (sigmatot, errtot);
  endpos = write_event_header_libxml2 (outFileName, "w");
  outFile = fopen (outFileName, "a");
  for (i = 0; i < nf; ++i) {
    infile[i] = freopen (filename[i], "r", infile[i]);
  }
  for (i = 0; i < nRecotot; ++i) {
    moveEvent (filename[file[i]], infile[file[i]], wpos[i], outFile, endpos);
    endpos = ftell(outFile);
  }

  fprintf (stdout,  "mix: file %s exhausted!\n", filename [final]);
  fprintf (stdout,  "mix: %i mixed/randomized events written to Mixed.PEV\n", nRecotot);
  fprintf (prtFile, "mix: %i mixed/randomized events written to Mixed.PEV\n", nRecotot);
  fclose (outFile);
  fclose (prtFile);
  {
    int filesize = 0;
    char check_sum[128];
    struct stat stt;
    char command[1024];

    stat (outFileName, &stt);
    filesize = stt.st_size;
    for (i = 0; i < 128; ++i) check_sum[i] = 0;
    sprintf (command, "md5sum %s > md5sum.log", outFileName);
    system(command);
    FILE * md5 = fopen ("md5sum.log", "r");
    if (md5) {
      fscanf (md5, "%s", check_sum);
    }
    unlink ("md5sum.log");
    set_final_numbers (nRecotot, filesize, sigmatot, errtot, check_sum);
    write_file_header_libxml2 (outFileName, "r+");
  }

  for (i = 0; i < nf; ++i) {
    free (filename[i]);
    free (pos[i]);
    free (map[i]);
  }
  free (filename);
  free (infile);
  free (pos);
  free (map);
  free (pos0);
  free (CSec);
  free (nLeft);
  free (ri);
  free (wpos);
  free (file);
  free (xmlerr);

  return 1;
}
